;************************************
;******************************************
;   This file is a basic code template for code generation on the  PIC16F726. *
;   This file contains the basic code building blocks to build upon.          *
;                                                                             *
;   Refer to the MPASM User's Guide for additional information on features of *
;   the assembler.                                                            *
;                                                                             *
;   Refer to the respective data sheet for additional                         *
;   information on the instruction set.                                       *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:         Sorting and Tally Counter.asm                          *
;    Date:              April 11, 2010
;	 REVISION MAY 14, 2010    
;	 REVISION MAY 28, 2011  WOULD NOT COMPILE WITH V8.66 (LABELS)             *
;    File Version:    C                                                       *
;                                                                             *
;    Author:    Ron Newton                                                    *
;    Company:                                                                 *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Files Required: P16F726.INC                                              *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Features of the 16F726:                                                  *
;                                                                             *
;    Precision Internal Oscillator (16 MHz or 500 kHz operation)              *
;    On Chip 3.2 V Regulator                                                  *
;    Pinout compatible to other 28/40 pin PIC16CXXX and 16FXXX micros         *
;    Brownout reset with selectable trip points                               *
;    8 bit resolution and 14 channel A/D Converter                            *
;    Wide operating voltage range (1.8-5.5V)                                  *
;    A/D converter with selectable 1.024, 2.048, 4.096 voltage reference      *
;    AUSART, SPI, and IIC                                                     *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Notes: THIS PROGRAM IS FOR A SORTING COUNTER AND TALLY COUNTER           *
;                                                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Revision History:                                                        *
;                                                                             *
;******************************************************************************

;------------------------------------------------------------------------------
; PROCESSOR DECLARATION
;------------------------------------------------------------------------------

     LIST      p=16F726             ; list directive to define processor
     #INCLUDE <P16F726.INC>         ; processor specific variable definitions

;------------------------------------------------------------------------------
;
; CONFIGURATION WORD SETUP
;
; The 'CONFIG' directive is used to embed the configuration word within the 
; .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration 
; word settings.
;
;------------------------------------------------------------------------------

     __CONFIG    _CONFIG1, _DEBUG_OFF & _PLL_EN & _BORV_1_9 & _BOR_OFF & _CP_OFF & _MCLR_EN & _PWRT_EN & _WDT_OFF & _INTOSCIO 
     __CONFIG    _CONFIG2, _VCAP_DIS

;------------------------------------------------------------------------------
;
; VARIABLE DEFINITIONS
;
; Available Data Memory divided into Bank 0 through Bank 3.  Each Bank contains
; Special Function Registers and General Purpose Registers at the locations 
; below:  
;
;           SFR           GPR               SHARED GPR's
; Bank 0    0x00-0x1F     0x20-0x6F         0x70-0x7F    
; Bank 1    0x80-0x9F     0xA0-0xEF         0xF0-0xFF  
; Bank 2    0x100-0x10F   0x110-0x16F       0x170-0x17F
; Bank 3    0x180-0x18F   0x190-0x1EF       0x1F0-0x1FF
;
;------------------------------------------------------------------------------
;PIN IN AND OUTPUTS	0 = OUT  1 = IN
;1	MCLR	NC				0
;2	RA0		SPEAKER			0
;3	RA1		DIGIT 1			0
;4	RA2		DIGIT 2			0
;5	RA3		DIGIT 3			0
;6	RA4		DIGIT 4			0
;7	RA5		NC				0
;8	VSS		GROUND
;9	RA7		NC				0
;10	RA6		NC				0
;11	RC0		F				0
;12	RC1		DECIMAL POINT	0
;13	RC2		C				0
;14	RC3		E				0
;15	RC4		B				0
;16	RC5		G				0
;17	RC6		D				0
;18	RC7		A				0
;19	VSS		GROUND
;20	VDD		6 VOLTS
;21	RB0		CLEAR	INT		1
;22	RB1		S1				1		
;23	RB2		S2				1
;24	RB3		S3				1
;25	RB4		S4				1
;26	RB5		S5				1
;27	RB6		MODE	SPCLK	1
;28	RB7		NC

#DEFINE			RESETB			PORTB,0		;RESET BUTTON
#DEFINE			S1				PORTB,1		;SWITCH 1
#DEFINE			S2				PORTB,2		;SWITCH 2
#DEFINE			S3				PORTB,3		;SWITCH 3
#DEFINE			S4				PORTB,4		;SWITCH 4
#DEFINE			S5				PORTB,5		;SWITCH 5
#DEFINE			MODE			PORTB,6		;MODE SWITCH

#DEFINE			MODE_FLAG   	FLAGS,0		;MODE FLAG = FLAGS 0	
#DEFINE			DECIMAL_FLAG	FLAGS,1		;DECIMAL FLAG = FLAG1
#DEFINE			SPEAKER			PORTA,0		;SPEAKER					
#DEFINE			DIGIT_ONE		PORTA,1		;DIGIT ONE IS PORT A 0		
#DEFINE			DIGIT_TWO		PORTA,2		;DIGIT TWO IS PORT A 1		
#DEFINE			DIGIT_THREE		PORTA,3		;DIGIT THREE IS PORT A 2	
#DEFINE			DIGIT_FOUR		PORTA,4		;DIGIT FOUR IS PORT A 3		

						
TOTAL_COUNT		EQU		20H			;TOTAL COUNT REGISTER LOW BYTE
TOTAL_COUNT1	EQU		21H			;TOTAL COUNT 1 REGISTER HIGH BYTE
COUNT1			EQU		22H			;COUNT 1 REGISTER
COUNT2			EQU		23H			;COUNT 2 REGISTER	
COUNT3			EQU		24H			;COUNT 3 REGISTER
COUNT4			EQU		25H			;COUNT 4 REGISTER
COUNT5			EQU		26H			;COUNT 5 REGISTER
TIMER1			EQU		27H			;ROUTINE COUNTER REGISTER
TIMER2			EQU		28H			;ROUTINE COUNTER REGISTER
TIMER3			EQU		29H			;ROUTINE COUNTER REGISTER
TIMER4			EQU		2AH			;ROUTINE COUNTER REGISTER
SECONDS			EQU		2BH			;SECONDS COUNTER REGISTER
MINUTES			EQU		2CH			;MINUTES COUNTER REGISTER
W_TEMP			EQU		2DH			;INTERRUPT HOLDING REGISTER
STATUS_TEMP		EQU		2EH			;INTERRUPT HOLDING REGISTER
PCLATH_TEMP		EQU		2FH			;INTERRUPT HOLDING REGISTER
FLAGS			EQU     30H			;INTERRUPT HOLDING REGISTER
MATH_HIGH		EQU		31H			;HEX TO DECIMAL HOLDING REGISTER
MATH_LOW		EQU		32H			;HEX TO DECIMAL HOLDING REGISTER	
TENK			EQU		33H			;HEX TO DECIMAL HOLDING REGISTER
THOUSANDS		EQU		34H			;THOUSANDS REGISTER	
HUNDREDS		EQU		35H			;HUNDREDS REGISTER
TENS			EQU		36H			;TENS REGISTER
ONES			EQU		37H			;ONES REGISTER
LOOP			EQU		38H			;ROUTINE COUNTER REGISTER		
LOOP1			EQU		39H			;ROUTINE COUNTER REGISTER
LOOP2			EQU		3AH			;ROUTINE COUNTER REGISTER
LOOP3			EQU		3BH			;ROUTINE COUNTER REGISTER
DELAY1			EQU		3CH			;ROUTINE COUNTER REGISTER
DELAY2			EQU		3DH			;ROUTINE COUNTER REGISTER
DEBOUNCE_TIMER 	EQU		3EH			;DEBOUNCE TIMER REGISTER
;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

RESET     ORG     0x0000            ; processor reset vector
            GOTO    AWAYWEGO        ; go to beginning of program

;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------

ISR       ORG     0x0004            ; interrupt vector location

;         Context saving for ISR
          MOVWF   W_TEMP            ; save off current W register contents
          MOVF    STATUS,W          ; move status register into W register
          MOVWF   STATUS_TEMP       ; save off contents of STATUS register
          MOVF    PCLATH,W          ; move pclath register into W register
          MOVWF   PCLATH_TEMP       ; save off contents of PCLATH register
		  BTFSC	  INTCON,T0IF		; timer 0 overflow?
		  GOTO	  ISR1              ; yes jump
	      BTFSC	  INTCON,INTF	    ; interrupt pin caused?
          GOTO	  ISRA			    ; yes jump
		  BTFSC	  INTCON,RBIF		; not check rbif
		  GOTO	  TIMER_CLEAR		; yes jump
ISR1      DECFSZ  TIMER1,1			; decrease timer 1
          GOTO    ISRA				; jump to isra
		  MOVLW	   .16				; loading timer 1
		  MOVWF	  TIMER1			; load
          DECFSZ  SECONDS,1			; decrease seconds
          GOTO 	  ISRA		    	; jump to isr2
          MOVLW   .60			    ; reload seconds
		  MOVWF	  SECONDS			; load
          DECFSZ  MINUTES,1			; decrease minutes
          GOTO    ISRA				; jump to isr2
          MOVLW   .1				; reload minutes
          MOVWF   MINUTES			; load
          BCF     INTCON,T0IE		; clear timer 0 flag
          MOVLW   .255				; load port a
          MOVWF    PORTA            ; turn off leds
          SLEEP	   					; put to sleep
		  NOP						; debugger stop

TIMER_CLEAR							; resetting
 	 	  MOVF	  PORTB,0			; reading port b to prevent interrupt
	      BCF	  INTCON,0			; clear portb interrupt flag
  		  MOVLW	   .16				; loading timer 1
		  MOVWF	  TIMER1			; load
	      MOVLW   .60				; make seconds 60  
	      MOVWF   SECONDS			; load 
		  MOVLW   .1				; make minutes 1 
          MOVWF   MINUTES			; load 
	
ISRA      BCF	  INTCON,T0IF		; clear timer 0 flag
		  BCF	  INTCON,INTF		; clear interrupt pin change
		  BCF	  INTCON,RBIF		; clear portb interrupt change
		  MOVLW	   B'11111000'		; loading intcon for all interrupts
		  MOVWF	   INTCON			; load
;         Restore context before returning from interrupt

          MOVF    PCLATH_TEMP,W     ; retrieve copy of PCLATH register
          MOVWF   PCLATH            ; restore pre-isr PCLATH register contents
          MOVF    STATUS_TEMP,W     ; retrieve copy of STATUS register
          MOVWF   STATUS            ; restore pre-isr STATUS register contents
          SWAPF   W_TEMP,F			; swapping
          SWAPF   W_TEMP,W          ; restore pre-isr W register contents
          RETFIE                    ; return from interrupt

;------------------------------------------------------------------------------
; MAIN PROGRAM
;------------------------------------------------------------------------------

AWAYWEGO							;LABEL
		BANKSEL	PORTA				;SELECTING SELECTING RIGHT BANK
		CLRF	PORTA				;CLEAR PORT A
		CLRF	PORTB				;CLEAR PORT B
		CLRF	PORTC				;CLEAR PORT C
		CLRF	PORTE				;CLEAR PORT E
		BANKSEL	TRISA				;SELECTING SELECTING RIGHT BANK
		CLRF	TRISA				;LOAD TRISA	0
		MOVLW	B'01000010'			;SETTING PORTC FOR IN AN OUT PORTS
		CLRF	TRISC				;LOAD TRISC 0
		MOVLW	B'01111111'			;ONE OUTPUT REST INPUT
		MOVWF	TRISB				;LOAD PORT B
		BANKSEL	OPTION_REG			;SELECTING SELECTING RIGHT BANK
		MOVLW	B'00000111'			;LOADING 3
		MOVWF	OPTION_REG			;LOAD
	    BANKSEL	WPUB				;SELECTING SELECTING RIGHT BANK
		MOVLW	B'01111111'			;MAKE WPUB INPUT WITH PULLUP
		MOVWF	WPUB				;LOAD
        BANKSEL OSCCON				;SELECTING RIGHT BANK
		MOVLW	B'00010000'			;4 MHZ INTERNAL
		MOVWF   OSCCON				;LOAD
		BANKSEL	ANSELB				;SELECTING RIGHT BANK
		CLRF	ANSELA				;CLEAR ANSELA
		CLRF	ANSELB				;CLEAR ANSELB
		BANKSEL	IOCB				;SELECTING RIGHT BANK
		MOVLW	B'01111111'			;MAKING RB0-RB6 INTERRUPTS
		MOVWF	IOCB				;LOAD
	    BANKSEL	PORTA				;SELECTING RIGHT BANK
	    MOVLW	.16					;LOADING TIMER 1
		MOVWF	TIMER1				;LOAD
		MOVLW	B'11111000'			;LOADING INTCON FOR ALL INTERRUPTS
		MOVWF	INTCON				;LOAD
	    MOVLW   .60					;MAKE SECONDS 60 
	    MOVWF   SECONDS				;LOAD
		MOVLW   .1					;MAKE MINUTES 1
        MOVWF   MINUTES				;LOAD
		CALL	RESETTING			;JUMP TO RESETTING AND RETURN
		MOVLW	.10					;LOADING TIMING LOOP 3
		MOVWF	LOOP3				;LOAD
;------------------------------------------------------------------------------
TESTING_AREA


;---------------------MODE SWITCH FOR CHANGING MODE----------------------------------------------
 
START	BTFSC	MODE				;CHECKING MODE SWITCH
		GOTO	SS4					;CLEAR TOGGLE
		CALL	DEBOUNCE			;DEBOUNCE SWITCH
		BTFSS	MODE				;WAIT UNTIL SWITCH IS RELEASE
		GOTO	$-1					;WAIT UNTIL SWITCH IS RELEASE
		CALL	RESETTING			;RESET COUNTERS
		BTFSC	MODE_FLAG			;CHECK MODE FLAG SKIP OF CLEAR
		GOTO	SS2					;NOT CLEAR JUMP
SS1		BSF		MODE_FLAG			;CLEAR SET
		GOTO	SS4					;JUMP TO SSR
SS2		BCF		MODE_FLAG			;NOT CLEAR, CLEAR
		
SS4		BTFSS	MODE_FLAG			;CHECK MODE FLAG
		GOTO	SS5					;JUMP TO SS5 (NOT SET)
		GOTO	SS6					;JUMP TO SS6 (SET)
SS5		GOTO	DIFFERENTIAL		;CLEARED JUMP TO DIFF
	
SS6		GOTO	ROUTINE_COUNT		;SET JUMP TO ROUTINE




DIFFERENTIAL	
		BCF		DECIMAL_FLAG		;TURN OFF DECIMAL POINT INDICATING MODE
		BTFSC	S1					;CHECK SWITCH ONE
		GOTO	D1					;NOT PUSHED JUMP
		CALL	DEBOUNCE			;DEBOUNCING TIMER
		BTFSS	S1					;ANTI BOUNCE
		GOTO	$-1					;ANTI BOUNCE
		INCF	TOTAL_COUNT,1		;INCREASE TOTAL COUNT
		INCF	COUNT1,1			;INCREASE COUNT1
D1		BTFSC	S2					;CHECK SWITCH TWO
		GOTO	D2					;NOT PUSHED JUMP
		CALL	DEBOUNCE			;DEBOUNCING TIMER
		BTFSS	S2					;ANTI BOUNCE
		GOTO	$-1					;ANTI BOUNCE
		INCF	TOTAL_COUNT,1		;INCREASE TOTAL COUNT
		INCF	COUNT2,1			;INCREASE COUNT2
D2		BTFSC	S3					;CHECK SWITCH THREE
		GOTO	D3					;NOT PUSHED JUMP
		CALL	DEBOUNCE			;DEBOUNCING TIMER
		BTFSS	S3					;ANTI BOUNCE
		GOTO	$-1					;ANTI BOUNCE
		INCF	TOTAL_COUNT,1		;INCREASE TOTAL COUNT
		INCF    COUNT3,1			;INCREASE COUNT3
D3		BTFSC	S4					;CHECK SWITCH FOUR
		GOTO	D4					;NOT PUSHED JUMP
		CALL	DEBOUNCE			;DEBOUNCING TIMER
		BTFSS	S4					;ANTI BOUNCE
		GOTO	$-1					;ANTI BOUNCE
		INCF	TOTAL_COUNT,1		;INCREASE TOTAL COUNT
		INCF	COUNT4,1			;INCREASE COUNT 4
D4		BTFSC	S5					;CHECK SWITCH 5
		GOTO	D5					;NOT PUSHED JUMP
		CALL	DEBOUNCE			;DEBOUNCING TIMER
		BTFSS	S5					;ANTI BOUNCE
		GOTO	$-1					;ANTI BOUNCE
		INCF	TOTAL_COUNT,1		;INCREASE TOTAL COUNT
		INCF	COUNT5,1			;INCREASE COUNT1
	
D5		MOVLW	.100				;CHECKING TO SEE TOTAL COUNT IS 100 YET
		SUBWF	TOTAL_COUNT,0		;CHECKING TO SEE TOTAL COUNT IS 100 YET
		BTFSC	3,2					;CHECKING TO SEE TOTAL COUNT IS 100 YET
		GOTO 	D6					;IS 100 JUMP
		MOVF	TOTAL_COUNT,0		;NOT YET
		MOVWF	MATH_LOW			;PREPARING TO DISPLAY COUNT
		MOVF	TOTAL_COUNT1,0		;PREPARING TO DISPLAY COUNT
		MOVWF	MATH_HIGH			;PREPARING TO DISPLAY COUNT
		CALL	CONVERSION			;CONVERT HEX TO DECIMAL
		CALL	DISPLAY				;DISPLAY TOTAL COUNT
		BTFSS	RESETB				;CHECK TO SEE IF RESET BUTTON PRESSED
		CALL	RESETTING			;YES RESET
		GOTO	START				;CONTINUE ON

D6		CALL	ALARM				;100 COUNT SOUND ALARM

;****************THIS SECTION DISPLAYS % OF COUNT********************

D6A		BTFSC	S1					;CHECK SWITCH 1
		GOTO	D7					;NOT PUSHED JUMP
		MOVF	COUNT1,0			;PREPARING TO DISPLAY COUNT
		MOVWF	MATH_LOW			;PREPARING TO DISPLAY COUNT
		CLRF	MATH_HIGH			;PREPARING TO DISPLAY COUNT
		CALL	CONVERSION			;CONVERT HEX TO DECIMAL
		CALL	DISPLAY				;DISPLAY COUNT1
		BTFSS	S1					;DISPLAY AS LONG AS SWITCH IS HELD
		GOTO	$-2					;DISPLAY AS LONG AS SWITCH IS HELD

D7		BTFSC	S2					;CHECK SWITCH 2
		GOTO	D8					;NOT PUSHED JUMP
		MOVF	COUNT2,0			;PREPARING TO DISPLAY COUNT
		MOVWF	MATH_LOW			;PREPARING TO DISPLAY COUNT
		CLRF	MATH_HIGH			;PREPARING TO DISPLAY COUNT
		CALL	CONVERSION			;CONVERT HEX TO DECIMAL
		CALL	DISPLAY				;DISPLAY COUNT2
		BTFSS	S2					;DISPLAY AS LONG AS SWITCH IS HELD
		GOTO	$-2					;DISPLAY AS LONG AS SWITCH IS HELD

D8		BTFSC	S3					;CHECK SWITCH 3
		GOTO	D9					;NOT PUSHED JUMP
		MOVF	COUNT3,0			;PREPARING TO DISPLAY COUNT
		MOVWF	MATH_LOW			;PREPARING TO DISPLAY COUNT
		CLRF	MATH_HIGH			;PREPARING TO DISPLAY COUNT
		CALL	CONVERSION			;CONVERT HEX TO DECIMAL
		CALL	DISPLAY				;DISPLAY 3
		BTFSS	S3					;DISPLAY AS LONG AS SWITCH IS HELD
		GOTO	$-2					;DISPLAY AS LONG AS SWITCH IS HELD

D9		BTFSC	S4					;CHECK SWITCH 4
		GOTO	D10					;NOT PUSHED JUMP
		MOVF	COUNT4,0			;PREPARING TO DISPLAY COUN
		MOVWF	MATH_LOW			;PREPARING TO DISPLAY COUNT
		CLRF	MATH_HIGH		    ;PREPARING TO DISPLAY COUNT
		CALL	CONVERSION			;CONVERT HEX TO DECIMAL
		CALL	DISPLAY				;DISPLAY 4
		BTFSS	S4					;DISPLAY AS LONG AS SWITCH IS HELD
		GOTO	$-2

D10		BTFSC	S5					;CHECK SWITCH 5
		GOTO	D11					;NOT PUSHED JUMP
		MOVF	COUNT5,0			;PREPARING TO DISPLAY COUNT
		MOVWF	MATH_LOW			;PREPARING TO DISPLAY COUNT
		CLRF	MATH_HIGH			;PREPARING TO DISPLAY COUNT
		CALL	CONVERSION			;CONVERT HEX TO DECIMAL
		CALL	DISPLAY				;DISPLAY 5
		BTFSS	S5					;DISPLAY AS LONG AS SWITCH IS HELD
		GOTO	$-2					;DISPLAY AS LONG AS SWITCH IS HELD
		
D11		MOVF	TOTAL_COUNT,0		;PREPARING TO DISPLAY TOTAL COUNT
		MOVWF	MATH_LOW			;PREPARING TO DISPLAY TOTAL COUNT
		CLRF	MATH_HIGH			;PREPARING TO DISPLAY TOTAL COUNT
		CALL	CONVERSION			;CONVERT HEX TO DECIMAL
		CALL	DISPLAY				;DISPLAY TOTAL COUNT
		BTFSC	RESETB				;CHECK IF RESET BUTTON IS PUSHED
		GOTO	D6A					;NOT PUSHED
		CALL	RESETTING			;PUSHED JUMP TO RESETTING AND RETURN
		GOTO	START				;GO TO START

RESETTING							;LABEL
		CLRF	TOTAL_COUNT			;CLEARING ALL COUNTERS
		CLRF	TOTAL_COUNT1		;CLEARING ALL COUNTERS
		CLRF	COUNT1				;CLEARING ALL COUNTERS
		CLRF	COUNT2				;CLEARING ALL COUNTERS
		CLRF	COUNT3				;CLEARING ALL COUNTERS
		CLRF	COUNT4				;CLEARING ALL COUNTERS
		CLRF	COUNT5				;CLEARING ALL COUNTERS
		CLRF	ONES				;CLEARING ALL COUNTERS
		CLRF	TENS				;CLEARING ALL COUNTERS
		CLRF	HUNDREDS			;CLEARING ALL COUNTERS
		CLRF	THOUSANDS			;CLEARING ALL COUNTERS
		CLRF	TENK
		CLRF	MATH_HIGH			;CLEARING ALL COUNTERS
		CLRF	MATH_LOW			;CLEARING ALL COUNTERS
		RETURN						;RETURN TO CALL

;****************TALLY COUNTER UP TO 9,999****************

;******************SINGLE BUTTON COUNT*****************
ROUTINE_COUNT						;LABEL
		BSF		DECIMAL_FLAG		;TURN ON MODE INDICATOR
		BTFSC	S1					;CHECK SWITCH 1
		GOTO	R_C1				;NOT PUSHED JUMP
		INCFSZ	TOTAL_COUNT,1		;INCREMENT TOTAL COUNT	
		GOTO	R_C0				;255 YET JUMP
		INCFSZ	TOTAL_COUNT1,1		;INCREASE TOTAL COUNT 1 TOTAL COUNT ROLLED OVER
R_C0	CALL	PULSE				;MAKE A CLICK AND RETURN
		CALL	DEBOUNCE			;DEBOUNCING TIMER
		BTFSS	S1					;ANTIBOUNCE
		GOTO	$-1					;ANTIBOUNCE

;****************COUNTING IN GROUPS OF THREES*******************
R_C1	BTFSC	S2					;CHECK SWITCH 2
		GOTO	R_C3				;NOT PUSHED JUMP
		MOVLW	3					;LOAD 3
		MOVWF	TIMER3				;LOAD 3 INTO TIMER 3
R_C1A	INCFSZ	TOTAL_COUNT,1		;INCREASE TOTAL COUNT
		GOTO	R_C1B				;CHECK FOR ROLL OEVER
		INCFSZ	TOTAL_COUNT1,1		;INCREASE TOTAL COUNT 1
R_C1B	CALL	PULSE				;MAKE A CLICK AND RETURN
		CALL	DELAY_ONE			;CLICK DELAY
		DECFSZ	TIMER3,1			;DO THREE TIMES
		GOTO	R_C1A				;JUMP
		CALL	DEBOUNCE			;DEBOUNCING TIMER
		BTFSS	S2					;WAIT FOR S2 TO RELEASE
		GOTO	$-1					;WAIT FOR S2 TO RELEASE

;*****************HOLDING BUTTON DOWN WILL PROVIDE CONTINOUS COUNT**************

R_C3	BTFSC	S3					;CHECK SWITCH 3
		GOTO	R_C4				;NOT PUSHED JUMP
		INCFSZ	TOTAL_COUNT,1		;INCREASE TOTAL COUNT
		GOTO	R_C3A				;CHECK FOR ROLL OVER
		INCF	TOTAL_COUNT1,1		;INCREASE TOTAL COUNT1
R_C3A	CALL	PULSE				;MAKE A CLICK AND RETURN
		CALL	DELAY_ONE			;CLICK DELAY
		CALL	DEBOUNCE			;DEBOUNCING TIMER
		BTFSS	S3					;CONTINUE COUNTING UNITL SWITCH IS RELEASED
		GOTO	R_C3				;CONTINUE COUNTING UNITL SWITCH IS RELEASED


R_C4	MOVF	TOTAL_COUNT,0		;PREPARING TO DISPLAY TOTAL COUNT AND TOTALCOUNT 1
		MOVWF	MATH_LOW			;PREPARING TO DISPLAY TOTAL COUNT AND TOTALCOUNT 1
		MOVF	TOTAL_COUNT1,0		;PREPARING TO DISPLAY TOTAL COUNT AND TOTALCOUNT 1
		MOVWF	MATH_HIGH			;PREPARING TO DISPLAY TOTAL COUNT AND TOTALCOUNT 1
		CALL	CONVERSION			;CONVERT HEX TO DECIMAL
		CALL	DISPLAY				;DISPLAY TOTAL COUNT
		BTFSC	RESETB				;CHECK RESET BUTTON		
		GOTO	START				;NOT PUSHED GO BACK TO START
		CALL	RESETTING			;PUSHED CLEAR ALL COUNTERS
		GOTO	START				;GO BACK TO START

	
PULSE   MOVLW		.50
		MOVWF		LOOP2
P1		MOVLW		.60				;LOADING NUMBER
		MOVWF		LOOP1			;LOADING NUMBER INTO LOOP1
		BCF			SPEAKER			;TURN ON SPEAKER
		DECFSZ		LOOP1,1			;TURN ON FOR A PERIOD OF TIME
		GOTO		$-1				;TURN ON FOR A PERIOD OF TIME
		MOVLW		.60				;LOADING NUMBER
		MOVWF		LOOP1			;LOADING NUMBER INTO LOOP1
		BSF			SPEAKER			;TURN OFF SPEAKER
		DECFSZ		LOOP1,1			;TURN OFF FOR A PERIOD OF TIME
		GOTO		$-1				;TURN OFF FOR A PERIOD OF TIME
		DECFSZ		LOOP2,1			;TURN OFF FOR A PERIOD OF TIME
		GOTO		P1				;TURN OFF FOR A PERIOD OF TIME	BCF		
		BSF			SPEAKER			;TURN OFF SPEAKER
		RETURN						;RETURN TO CALL

ALARM	BSF			DIGIT_ONE		;TURN OFF DISPLAY
		BSF			DIGIT_TWO		;TURN OFF DISPLAY
		BSF			DIGIT_THREE		;TURN OFF DISPLAY
		BSF			DIGIT_FOUR		;TURN OFF DISPLAY

A1		MOVLW		.60				;LOADING NUMBER
		MOVWF		LOOP1			;LOADING NUMBER INTO LOOP1
		BCF			SPEAKER			;TURN ON SPEAKER
		DECFSZ		LOOP1,1			;TURN ON FOR A PERIOD OF TIME
		GOTO		$-1				;TURN ON FOR A PERIOD OF TIME
		MOVLW		.60				;LOADING NUMBER
		MOVWF		LOOP1			;LOADING NUMBER INTO LOOP1
		BSF			SPEAKER			;TURN OFF SPEAKER
		DECFSZ		LOOP1,1			;TURN OFF FOR A PERIOD OF TIME
		GOTO		$-1				;TURN OFF FOR A PERIOD OF TIME
		DECFSZ		LOOP2,1			;SOUND SPEAKER FOR A PERIOD OF TIME	
		GOTO		ALARM			;SOUND SPEAKER FOR A PERIOD OF TIME
		DECFSZ		LOOP3,1			;ALARM FOR SECONDS
		GOTO	    A1 				;JUMP TO ALARM FOR 2 SECONDS
		MOVLW		.10				;RELOADING LOOP3
		MOVWF		LOOP3			;LOAD
		RETURN						;RETURN TO CALL

;****************** THIS CONVERTS HEX TO DECIMAL *****************
CONVERSION							;LABEL
		SWAPF		MATH_HIGH,W		;SWAP THE FIRST FOUR BITS WITH THE LAST FOUR BITS (NIBBLES)
		ANDLW		0FH				;GET RID OF THE LAST SWAPPED DIGITS
		ADDLW		0F0H			;ADD 240 TO RESULT
		MOVWF		THOUSANDS		;MOVE INTO THOUSANDS
		ADDWF		THOUSANDS,F		;ADD THOUSANDS TO W AND PUT RESULT IN THOUSANDS 
		ADDLW		0E2H			;ADD 226
		MOVWF		HUNDREDS		;MOVE W INTO HUNDREDS 
		ADDLW		32H				;ADD 50 TO W
		MOVWF		ONES			;MOVE W INTO ONES

		MOVF		MATH_HIGH,0		;MOVE MATH HIGH INTO W
		ANDLW		0FH				;GET RID OF LAST 4 BITS
		ADDWF		HUNDREDS,F		;ADD W TO HUNDREDS
		ADDWF		HUNDREDS,F		;ADD AGAIN
		ADDWF		ONES,F			;ADD W TO ONES 
		ADDLW		0E9H			;ADD 233
		MOVWF		TENS			;MOVE W INTO TENS 
		ADDWF		TENS,F			;ADD W TO TENS
		ADDWF		TENS,F			;ADD AGAIN

		SWAPF		MATH_LOW,W		;SWAP NIBBLES OF MATH LOW
		ANDLW		0FH				;GET RID OF HIGH NIBBLES
		ADDWF		TENS,F			;ADD W TO TENS 
		ADDWF		ONES,F			;ADD W TO ONES

		RLF			TENS,F			;ROTATE TO THE LEFT TENS
		RLF			ONES,F			;ROTATE TO THE LEFT ONES		
		COMF		ONES,F			;COMPLEMENT ONES
		RLF			ONES,F			;ROTATE TO THE LEFT ONES
			
		MOVF		MATH_LOW,W		;MOVE MATH LOW INTO W
		ANDLW		0FH				;GET RID OF FIRST NIBBLE
		ADDWF		ONES,F			;ADD W TO ONES 
		RLF			THOUSANDS,F		;ROTATE LEFT THOUSANDS 
		
		MOVLW		07H				;MOVE 7 INTO TENK
		MOVWF		TENK			;LOAD

		MOVLW		0AH				;MOVE 10 INTO W
LB1		ADDWF		ONES,F			;ADD 10 TO ONES
		DECF		TENS,F			;DECREASE TENS
		BTFSS		3,0				;CHECK FOR A BARROW OR CARRY
		GOTO		LB1				;DO UNTIL BARROW
LB2		ADDWF		TENS,F			;ADD W TO TENS
		DECF		HUNDREDS,F		;DECREASE HUNDREDS
		BTFSS		3,0				;CHECK FOR BARROW OR CARRY
		GOTO		LB2				;DO UNTIL BARROW

LB3		ADDWF		HUNDREDS,F		;ADD W TO HUNDREDS
		DECF		THOUSANDS,F		;DECREASE THOUSANDS
		BTFSS		3,0				;CHECK FOR BARROW OR CARRY
		GOTO		LB3				;DO UNTIL BARROW

LB4		ADDWF		THOUSANDS,F		;ADD W TO THOUSANDS
		DECF		TENK,F			;DECREASE TENK
		BTFSS		3,0				;CHECK FOR BARROW OR CARRY
		GOTO		LB4				;DO UNTIL BARROW

		RETURN						;RETURN TO CALL

DISPLAY								;LABEL
;**********************ONES***********************************
		BSF			INTCON,GIE		;STOP INTERRUPTS
		BSF			DIGIT_ONE		;TURN OFF DIGIT ONE
		BSF			DIGIT_THREE		;TURN OFF DIGIT THREE
		BSF			DIGIT_FOUR		;TURN OFF DIGIT FOUR
		BSF			DIGIT_TWO		;TURN OFF DIGIT TWO
		MOVLW		01H				;LOAD INTO ADDR THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS 
		ADDWF		ONES,0			;ADD THE OFFSET TO THE LOW ADDRESS (ADDR)
		MOVLW		.2				;LOAD 8 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC		STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW		1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF		PCLATH			;MOVE ONE INTO PCLATH REGISTER
		MOVF		ONES,0			;LOAD OFFSET INTO W
		CALL		PATTERN			;JUMP TO PATTERN
		CLRF		PCLATH			;RESET PCLATH
		MOVWF		PORTC			;MOVE PATTERN INTO PORTC
		BTFSC		DECIMAL_FLAG	;CHECK FOR MODE
		BCF			PORTC,1			;TURN ON DECIMAL POINT	GOT
		BCF			DIGIT_ONE		;TURN ON DIGIT ONE
		CALL		DELAY			;CALL THIS LABEL. THIS DELAY ALLOWS FOR BETTER DISPLAYING

;************************TENS*****************************************
		MOVLW		0				;TURNING OFF UN-NECESSARY DIGITS
		SUBWF		TENS,0
		BTFSC		3,2
		GOTO		TA
		GOTO		T1
TA		MOVLW		0
		SUBWF		HUNDREDS,0
		BTFSC		3,2
		GOTO		TB
		GOTO		T1
TB		MOVLW		0
		SUBWF		THOUSANDS,0
		BTFSC		3,2
		GOTO		BACK		
		
T1		BSF			DIGIT_ONE		;TURN OFF DIGIT ONE
		BSF			DIGIT_THREE		;TURN OFF DIGIT THREE
		BSF			DIGIT_TWO		;TURN OFF DIGIT TWO
		BSF			DIGIT_FOUR		;TURN OFF DIGIT FOUR
		MOVLW		01H				;LOAD INTO ADDR THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS 
		ADDWF		TENS,0			;ADD THE OFFSET TO THE LOE ADDRESS (ADDR)
		MOVLW		.2				;LOAD 8 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC		STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW		1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF		PCLATH			;MOVE ONE INTO PCLATH REGISTER
		MOVF		TENS,0			;LOAD OFFSET INTO W
		CALL 		PATTERN			;CALL PATTERN AND RETURN
		CLRF		PCLATH			;RESET PCLATH
		MOVWF		PORTC			;MOVE WHAT WAS IN PATTERN TO PORT B
		BCF			DIGIT_TWO		;TURN ON DIGIT TWO
		CALL		DELAY			;CALL THIS LABEL. THIS DELAY ALLOWS FOR BETTER DISPLAYING		
		BSF			PORTC,1			;TURN OFF DECIMAL

;*********************HUNDREDS**********************************	
		MOVLW		0				;TURNING OFF UN-NECESSARY DIGITS
		SUBWF		HUNDREDS,0
		BTFSC		3,2
		GOTO		HA
		GOTO		H1
HA		MOVLW		0
		SUBWF		THOUSANDS,0
		BTFSC		3,2
		GOTO		BACK		

H1		BSF			DIGIT_ONE		;TURN OFF DIGIT ONE
		BSF			DIGIT_TWO		;TURN OFF DIGIT TWO
		BSF			DIGIT_THREE		;TURN OFF DIGIT THREE
		BSF			DIGIT_FOUR		;TURN OFF DIGIT FOUR
		MOVLW		01H				;LOAD INTO ADDR THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS 
		ADDWF		HUNDREDS,0		;ADD THE OFFSET TO THE LOW ADDRESS (ADDR)
		MOVLW		.2				;LOAD 8 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC		STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW		1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF		PCLATH			;MOVE ONE INTO PCLATH REGISTER
		MOVF		HUNDREDS,0		;LOAD OFFSET INTO W
		CALL 		PATTERN			;CALL PATTERN AND RETURN
		CLRF		PCLATH			;RESET PCLATH
		MOVWF		PORTC			;MOVE WHAT WAS IN PATTERN TO PORT C
		BCF			DIGIT_THREE		;TURN ON DIGIT THREE
		CALL		DELAY			;CALL THIS LABEL. THIS DELAY ALLOWS FOR BETTER DISPLAYING		
		BSF			PORTC,1			;TURN OFF DECIMAL

;************************THOUSANDS******************************

		MOVLW		0				;TURNING OFF UN-NECESSARY DIGITS
		SUBWF		THOUSANDS,0
		BTFSC		3,2
		GOTO		BACK		

TH1		BSF			DIGIT_ONE		;TURN OFF DIGIT ONE
		BSF			DIGIT_TWO		;TURN OFF DIGIT TWO
		BSF			DIGIT_THREE		;TURN OFF DIGIT THREE
		BSF			DIGIT_FOUR		;TURN OFF DIGIT FOUR
		MOVLW		01H				;LOAD INTO ADDR THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS 
		ADDWF		THOUSANDS,0		;ADD THE OFFSET TO THE LOW ADDRESS (ADDR)
		MOVLW		.2				;LOAD 8 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC		STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW		1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF		PCLATH			;MOVE ONE INTO PCLATH REGISTER
		MOVF		THOUSANDS,0		;LOAD OFFSET INTO W
		CALL 		PATTERN			;CALL PATTERN AND RETURN
		CLRF		PCLATH			;RESET PCLATH
		MOVWF		PORTC			;MOVE WHAT WAS IN PATTERN TO PORT C
		BCF			DIGIT_FOUR		;TURN ON DIGIT FOUR
		CALL		DELAY			;CALL THIS LABEL. THIS DELAY ALLOWS FOR BETTER DISPLAYING		
		BSF			DIGIT_FOUR		;TURN OFF DIGIT FOUR
BACK	BCF			INTCON,GIE		;TURN ON INTERRUPTS
		RETURN						;RETURN TO CALL

DEBOUNCE							;LABEL
		DECFSZ 		DEBOUNCE_TIMER,1	;ADDING DEBOUNCE TIME
		GOTO		$-1					;ADDING DEBOUNCE TIME
		RETURN	
DELAY								;LABEL
		MOVLW		.2				;DISPLAY DELAY
		MOVWF		DELAY1			;DISPLAY DELAY
		DECFSZ		DELAY2,1		;DISPLAY DELAY
		GOTO		$-1				;DISPLAY DELAY
		DECFSZ		DELAY1,1		;DISPLAY DELAY
		GOTO		$-3				;DISPLAY DELAY
		RETURN						;RETURN

DELAY_ONE							;LABEL
		MOVLW		.100			;CLICK DELAY
		MOVWF		DELAY1			;CLICK DELAY
		DECFSZ		DELAY2,1		;CLICK DELAY
		GOTO		$-1				;CLICK DELAY
		DECFSZ		DELAY1,1		;CLICK DELAY
		GOTO		$-3				;CLICK DELAY
		RETURN						;RETURN


;*************************THIS AREA IS A LOOK UP TABLE FOR NUMBERS***********

		ORG	200H				;SET AT EVEN PAGE
PATTERN							;LABEL 

		ADDWF	PCL,1			;ADD W TO PROGRAM COUNTER AND LOOK UP DATA
		RETLW	B'00100010'		;0 DISPLAY
		RETLW	B'11101011'		;1 DISPLAY
		RETLW	B'00000111'		;2 DISPLAY
		RETLW	B'00001011'		;3 DISPLAY
		RETLW	B'11001010'		;4 DISPLAY
		RETLW	B'00011010'		;5 DISPLAY
		RETLW	B'10010010'		;6 DISPLAY
		RETLW	B'01101011'		;7 DISPLAY
		RETLW	B'00000010'		;8 DISPLAY
		RETLW	B'01001010'		;9 DISPLAY



          END
